/* Based on FreeRTOS Demo main (AVR-GCC Port). */

#include <stdlib.h>
#include <string.h>
#include <avr/eeprom.h> 
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

/* Scheduler include files. */
#include "FreeRTOS.h"
#include "task.h"
#include "croutine.h"

#include "status.h"
#include "uart.h"

/* Priority definitions for most of the tasks in the demo application.  Some
tasks just use the idle priority. */
#define mainCHECK_TASK_PRIORITY			( tskIDLE_PRIORITY + 3 )

/* Baud rate used by the serial port tasks. */
#define UART_BAUD_RATE			( ( unsigned portLONG ) 9600 )

/* define CPU frequency in Mhz here if not defined in Makefile */
#ifndef F_CPU
#define F_CPU 16000000UL
#endif

/* LED that is toggled by the check task.  The check task periodically checks
that all the other tasks are operating without error.  If no errors are found
the LED is toggled.  If an error is found at any time the LED is never toggles
again. */
#define mainCHECK_TASK_LED				( 7 )

/* The period between executions of the check task. */
#define mainCHECK_PERIOD				( ( portTickType ) 3000 / portTICK_RATE_MS  )

/*
 * Function prototypes, see below.
 */
static void serialWriter( void *pvParameters );
static void vErrorChecks( void *pvParameters );
static void prvCheckOtherTasksAreStillRunning( void );
void vApplicationIdleHook( void );

/*
 * status variables.
 */
static portBASE_TYPE serialWriterStatus = pdTRUE;

portSHORT main( void ) {
  /* Setup the LED's for output. */
  statusInitialize();
  uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU) ); 
  sei();

  /* Create the tasks defined within this file. */
  xTaskCreate( vErrorChecks, ( signed portCHAR * ) "Check", configMINIMAL_STACK_SIZE, NULL, mainCHECK_TASK_PRIORITY, NULL );
  xTaskCreate( serialWriter, ( signed portCHAR * ) "Foo", configMINIMAL_STACK_SIZE, "foo", mainCHECK_TASK_PRIORITY, NULL );
  xTaskCreate( serialWriter, ( signed portCHAR * ) "Bar", configMINIMAL_STACK_SIZE, "bar", mainCHECK_TASK_PRIORITY, NULL );
  uart_puts_P("\r\nFreeRTOS template initialized.");

  vTaskStartScheduler();
  return 0;
}

static void vErrorChecks( void *pvParameters ) {
  /* The parameters are not used. */
  ( void ) pvParameters;

  /* Cycle for ever, delaying then checking all the other tasks are still
     operating without error. */
  for( ;; ) {
    vTaskDelay( mainCHECK_PERIOD );
    prvCheckOtherTasksAreStillRunning();
  }
}

static void serialWriter( void *pvParameters ) {
  char* message = pvParameters;
  for (;;) {
	vTaskDelay((portTickType) 100/portTICK_RATE_MS);
	/* Notify a single error. */
	if (uart_puts_P("\r\n") != pdTRUE)
	  serialWriterStatus = pdFALSE;
	if (uart_puts(message) != pdTRUE)
	  serialWriterStatus = pdFALSE;
  }
}

static void prvCheckOtherTasksAreStillRunning( void ) {
  static portBASE_TYPE xErrorHasOccurred = pdFALSE;
  //if( xCheckSerialTasks() != pdTRUE ) {
  //  xErrorHasOccurred = pdTRUE;
  //}

  if( xErrorHasOccurred == pdFALSE ) {
    /* Toggle the LED if everything is okay so we know if an error occurs even if not
       using console IO. */
    statusToggleLED( mainCHECK_TASK_LED );
    uart_puts_P("\r\nAll tasks working.");
  }
}

void vApplicationIdleHook( void ) {
  vCoRoutineSchedule();
}

